'Active DC heater control for common 9/12 with logical-hysteresis using picAxe08M.
'---------------------------------------------------------------------------------
'Because every turbine has different power characteristics, an active method of load-adjustment
'at different speeds is required. With only two inputs available, and one used to monitor
'RPM, the high resolution analog to digital(ADC 10-bit) input of AXE08M conveniently gives a  
'0-1023 step conversion of a analog potentiometer spanning the controllers supply rails, allowing
'adjustment, and fine tuning, of the load engagements for different sized turbines. This manual
'adjustment will result in an actively increasing load window starting at 150rpm to 400rpm(for small
'turbines), maintaining system control(tames the turbine) until full-furl.
'
'The prototype includes four heater elements of 3ohms nichrome wire, each. Note, different element
'loads and combinations may result in more efficient harvesting of power, however, do not
'exceed 'total-equivalent parallel resistance' less than the alternators single-phase resistance.
'The math routines in this program are compatible with 12-pole 3phase alternators only.
'Ver1.3 by CompDoc, Jan 2007.

; Variable-registers 
SYMBOL BaseRef = b0  ; Starting point for engaging additional loads/elements.
SYMBOL RPM = b5  ; RPS is revolutions per second. RPM=RPSx10 for 9-12 machine.
SYMBOL Mode = b6  ; used to keep track of last heater mode for +/-50rpm hysteresis.

Mode=0
let dirs = %00000111  ; three outputs drive power-NMosFets(T220 package).

Start:
; Determine engage points to actively load the turbine and harvest power.
readadc10 4,w6 ; Manual adjustment reference point(0-255) from pot-wiper on pin4.
BaseRef=w6/40  ; Convert base reference to 0-25 steps/increments (quotient).
b1=BaseRef+15
b2=BaseRef+20
b3=BaseRef+25
b4=BaseRef+30

;continuously monitor changes in turbine speed and adjust load to track available power.
Count 3,1000,RPM ; count pulses from one phase(pin3) for 1 second(1/10th rpm).
	; RPM equals pulses per second x 10 for the 9-12 3phase Alternators.
	if RPM < b1 then RunMin
	if RPM >= b4 then RunHigh
	if RPM >= b3 and Mode < 3 then RunMed
	if RPM >= b3 and Mode >= 3 then RunHigh
	if RPM >= b2 and Mode < 2 then RunLow
	if RPM >= b2 and Mode >= 2 then RunMed
	if RPM >= b1 and Mode < 1 then RunMin
	if RPM >= b1 and Mode >= 1 then RunLow
	goto Start ; else loop back to start

RunMin:
; run heater in minimum mode (production system=3ohm load)
let Mode=0
let pins = %00000111 ; switch all outputs off
goto Start ; loop back to start

RunLow:
; run heater in low mode (two-3ohm elements in parallel=1.5ohm load)
let Mode=1
let pins = %00000110 ; switch outputs Q0 on
goto Start ; loop back to start

RunMed:
; run heater in medium mode (three-3ohm elements in parallel=1ohm load)
let Mode=2
let pins = %00000100 ; switch outputs Q0,1 on
goto Start ; loop back to start

RunHigh:
; run heater in High mode (four-3ohm elements in parallel=0.75ohm load)
let Mode=3
let pins = %00000000 ; switch outputs Q0,1,2 on
goto Start ; loop back to start

end


